package dev.shadowsoffire.apotheosis.adventure.affix;

import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.CommonComponents;
import net.minecraft.resources.ResourceLocation;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import net.minecraft.world.level.storage.loot.LootContext;
import it.unimi.dsi.fastutil.objects.ObjectArrayList;
import net.minecraft.world.item.enchantment.Enchantment;
import java.util.Map;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.world.level.block.state.BlockState;
import net.minecraft.core.BlockPos;
import net.minecraft.world.level.LevelAccessor;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.phys.HitResult;
import net.minecraft.world.InteractionResult;
import net.minecraft.world.item.context.UseOnContext;
import net.minecraft.world.entity.projectile.AbstractArrow;
import javax.annotation.Nullable;
import net.minecraft.world.entity.Entity;
import net.minecraft.world.entity.LivingEntity;
import net.minecraft.world.entity.MobType;
import net.minecraft.world.damagesource.DamageSource;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.MutableComponent;
import net.minecraft.world.entity.ai.attributes.AttributeModifier;
import net.minecraft.world.entity.ai.attributes.Attribute;
import java.util.function.BiConsumer;
import net.minecraft.world.entity.EquipmentSlot;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.placebo.codec.CodecProvider;

public abstract class Affix implements CodecProvider<Affix>
{
    protected final AffixType type;
    
    public Affix(final AffixType type) {
        this.type = type;
    }
    
    public void addModifiers(final ItemStack stack, final LootRarity rarity, final float level, final EquipmentSlot type, final BiConsumer<Attribute, AttributeModifier> map) {
    }
    
    public MutableComponent getDescription(final ItemStack stack, final LootRarity rarity, final float level) {
        return Component.m_237110_("affix." + this.getId() + ".desc", new Object[] { fmt(level) });
    }
    
    public Component getName(final boolean prefix) {
        if (prefix) {
            return (Component)Component.m_237115_("affix." + this.getId());
        }
        return (Component)Component.m_237115_("affix." + this.getId() + ".suffix");
    }
    
    public Component getAugmentingText(final ItemStack stack, final LootRarity rarity, final float level) {
        return (Component)this.getDescription(stack, rarity, level);
    }
    
    public int getDamageProtection(final ItemStack stack, final LootRarity rarity, final float level, final DamageSource source) {
        return 0;
    }
    
    public float getDamageBonus(final ItemStack stack, final LootRarity rarity, final float level, final MobType creatureType) {
        return 0.0f;
    }
    
    public void doPostAttack(final ItemStack stack, final LootRarity rarity, final float level, final LivingEntity user, @Nullable final Entity target) {
    }
    
    public void doPostHurt(final ItemStack stack, final LootRarity rarity, final float level, final LivingEntity user, @Nullable final Entity attacker) {
    }
    
    public void onArrowFired(final ItemStack stack, final LootRarity rarity, final float level, final LivingEntity user, final AbstractArrow arrow) {
    }
    
    @Nullable
    public InteractionResult onItemUse(final ItemStack stack, final LootRarity rarity, final float level, final UseOnContext ctx) {
        return null;
    }
    
    public void onArrowImpact(final AbstractArrow arrow, final LootRarity rarity, final float level, final HitResult res, final HitResult.Type type) {
    }
    
    public float onShieldBlock(final ItemStack stack, final LootRarity rarity, final float level, final LivingEntity entity, final DamageSource source, final float amount) {
        return amount;
    }
    
    public void onBlockBreak(final ItemStack stack, final LootRarity rarity, final float level, final Player player, final LevelAccessor world, final BlockPos pos, final BlockState state) {
    }
    
    public float getDurabilityBonusPercentage(final ItemStack stack, final LootRarity rarity, final float level, @Nullable final ServerPlayer user) {
        return 0.0f;
    }
    
    public float onHurt(final ItemStack stack, final LootRarity rarity, final float level, final DamageSource src, final LivingEntity ent, final float amount) {
        return amount;
    }
    
    public boolean enablesTelepathy() {
        return false;
    }
    
    public void getEnchantmentLevels(final ItemStack stack, final LootRarity rarity, final float level, final Map<Enchantment, Integer> enchantments) {
    }
    
    public void modifyLoot(final ItemStack stack, final LootRarity rarity, final float level, final ObjectArrayList<ItemStack> loot, final LootContext ctx) {
    }
    
    @Override
    public String toString() {
        return String.format("Affix: %s", this.getId());
    }
    
    public AffixType getType() {
        return this.type;
    }
    
    public abstract boolean canApplyTo(final ItemStack p0, final LootCategory p1, final LootRarity p2);
    
    public static boolean isOnCooldown(final ResourceLocation id, final int cooldown, final LivingEntity entity) {
        final long lastApplied = entity.getPersistentData().m_128454_("apoth.affix_cooldown." + id.toString());
        return lastApplied != 0L && lastApplied + cooldown >= entity.m_9236_().m_46467_();
    }
    
    public static void startCooldown(final ResourceLocation id, final LivingEntity entity) {
        entity.getPersistentData().m_128356_("apoth.affix_cooldown." + id.toString(), entity.m_9236_().m_46467_());
    }
    
    public static String fmt(final float f) {
        if (f == (long)f) {
            return String.format("%d", (long)f);
        }
        return ItemStack.f_41584_.format(f);
    }
    
    public static MutableComponent valueBounds(final Component min, final Component max) {
        return CommonComponents.m_264333_().m_7220_((Component)Component.m_237110_("misc.apotheosis.affix_bounds", new Object[] { min, max }).m_130940_(ChatFormatting.DARK_GRAY));
    }
    
    public final ResourceLocation getId() {
        return AffixRegistry.INSTANCE.getKey((CodecProvider)this);
    }
}
